# -*- coding: iso-8859-1 -*-
"""
    GESTIKK
    Version 0.5

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007-2008:
    Friedrich 'fred.reichbier' Weber (2007-10-08) and
    Jonas 'Dauerbaustelle' Haag (2008)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""

import gtk
import pango

class Window():
    def __init__(self, color='red', line_width=5):
        """
            :param color: The foreground color we want to use. can be named ('black', 'red', etc.) or hex ('#ff0000', '#abcdef')
            :param line_width: The width of a line
        """
        # get screen
        self.screen = gtk.gdk.screen_get_default()
        # get size
        self.size = (self.screen.get_width(), self.screen.get_height())
        # get the colormap and the fg color
        self.colormap = gtk.gdk.colormap_get_system()
        self.color = self.colormap.alloc_color(color)
        self.c_black = self.colormap.alloc_color('black')
        self.c_white = self.colormap.alloc_color('white')
        # create the window (without decorations and display in the taskbar through override_redirect)
        self.window = gtk.gdk.Window(None, self.size[0], self.size[1], gtk.gdk.WINDOW_TOPLEVEL, gtk.gdk.EXPOSURE_MASK, gtk.gdk.INPUT_OUTPUT)
        self.window.set_decorations(0)
        self.window.set_override_redirect(True)
        # create the pixmap
        self.pixmap = gtk.gdk.Pixmap(self.window, self.size[0], self.size[1])
        self.pixmap.gc = self.pixmap.new_gc(foreground=self.color, line_width=line_width)
        # create the mask
        self.mask = gtk.gdk.Pixmap(None, self.size[0], self.size[1], 1)
        self.mask.gc = self.mask.new_gc(foreground=self.c_white, line_width=line_width)
        # reset everything
        self.clear()
        # set `old_pos`
        self.old_pos = None
        # and - showtime!
        self.window.stick()
        self.window.show()

    def destroy(self):
        """ destroy the window """
        self.window.destroy()

    def paint_gesture_path(self, x, y):
        """ paint a new station in the gesture's path """
        if self.old_pos:
            self.draw_line(self.old_pos[0], self.old_pos[1], x, y)
        self.old_pos = (x, y)

    def clear(self):
        """ clear + remove all """
        self.reset_mask()
        self.reset_pixmap()
        self.update_mask()
        self.update_pixmap()
        self.old_pos = None

    def draw_line(self, x1, y1, x2, y2):
        """ wrapper for a line """
        # draw on pixmap
        self.pixmap.draw_line(self.pixmap.gc, x1, y1, x2, y2)
        # draw on mask
        self.mask.draw_line(self.mask.gc, x1, y1, x2, y2)
        # update all
        self.update_mask()
        self.update_pixmap()

    def reset_mask(self):
        """ reset the shape mask to fully transparent """
        # fill black
        self.mask.gc.set_foreground(self.c_black) #todo?
        self.mask.draw_rectangle(self.mask.gc, True, 0, 0, self.size[0], self.size[1])
        self.mask.gc.set_foreground(self.c_white)

    def reset_pixmap(self):
        """ reset the pixmap to fully black """
        # fill black
        self.pixmap.gc.set_foreground(self.c_black) #todo?
        self.pixmap.draw_rectangle(self.pixmap.gc, True, 0, 0, self.size[0], self.size[1])
        self.pixmap.gc.set_foreground(self.color)

    def update_mask(self):
        """ update the mask """
        self.window.shape_combine_mask(self.mask, 0, 0) # vllt nach .show()?

    def update_pixmap(self):
        """ update the pixmap """#00006c53ffff
        self.window.set_back_pixmap(self.pixmap, False)

class OSDNotifier():
    def __init__(self, color='red', f="Sans", fs=20):
        """
            :param color: The foreground color
        """
        #save vars
        self.font_name = f
        self.font_size = int(fs)
        #set right size of font
        self.font_size = self.font_size * 1000
        # get screen
        self.screen = gtk.gdk.screen_get_default()
        # get position and size
        self.position = (10, 10)
        self.size = (self.screen.get_width(), self.screen.get_height()) # todo: do we really need the whole screen?
        # get colormap
        self.colormap = gtk.gdk.colormap_get_system()
        self.color = self.colormap.alloc_color(color)
        self.c_black = self.colormap.alloc_color('black')
        self.c_white = self.colormap.alloc_color('white')

        self.window = gtk.gdk.Window(None, self.size[0], self.size[1], gtk.gdk.WINDOW_TOPLEVEL, gtk.gdk.EXPOSURE_MASK, gtk.gdk.INPUT_OUTPUT)
        self.window.set_decorations(0)
        self.window.set_override_redirect(True)
        # create pango renderer + context for text
        self.pango_context = gtk.gdk.pango_context_get_for_screen(self.screen)
        self.pango_layout = pango.Layout(self.pango_context)
        # create the pixmap
        self.pixmap = gtk.gdk.Pixmap(self.window, self.size[0], self.size[1])
        self.pixmap.gc = self.pixmap.new_gc(foreground=self.color)
        # create the mask
        self.mask = gtk.gdk.Pixmap(None, self.size[0], self.size[1], 1)
        self.mask.gc = self.mask.new_gc(foreground=self.c_white)
        self.mask.gc.set_colormap(self.colormap)
        self.mask.gc.set_foreground(self.c_white)
        # clear
        self.clear()
        # and show on all workspaces
        self.window.show()
        self.window.stick()

    def notify(self, s, gesture):
        """ Display `s`; `gesture` is the repr-ed gesture. no need for this at the moment """
        # clear
        self.clear()
        # set pango text
        self.pango_layout.set_markup('<span face="%s" weight="bold" size="%s">%s</span>' % (self.font_name, self.font_size, s))
        # get size and calculate position
        size = self.pango_layout.get_pixel_size()
        self.position = (10, self.screen.get_height() - size[1] - 40)
        # draw on pixmap and mask
        self.pixmap.draw_layout(self.pixmap.gc, self.position[0], self.position[1], self.pango_layout)
        self.mask.draw_layout(self.mask.gc, self.position[0], self.position[1], self.pango_layout)
        # update everything
        self.update_pixmap()
        self.update_mask()

    def clear(self):
        """ clear + remove all """
        self.reset_mask()
        self.reset_pixmap()
        self.update_mask()
        self.update_pixmap()

    def reset_mask(self):
        """ reset the shape mask to fully transparent """
        # fill black
        self.mask.gc.set_foreground(self.c_black) #todo?
        self.mask.draw_rectangle(self.mask.gc, True, 0, 0, self.size[0], self.size[1])
        self.mask.gc.set_foreground(self.c_white)

    def reset_pixmap(self):
        """ reset the pixmap to fully black """
        # fill black
        self.pixmap.gc.set_foreground(self.c_black) #todo?
        self.pixmap.draw_rectangle(self.pixmap.gc, True, 0, 0, self.size[0], self.size[1])
        self.pixmap.gc.set_foreground(self.color)

    def destroy(self):
        """ destroy the window """
        self.window.destroy()

    def update_mask(self):
        """ update the mask """
        self.window.shape_combine_mask(self.mask, 0, 0)

    def update_pixmap(self):
        """ update the pixmap """
        self.window.set_back_pixmap(self.pixmap, False)

